unit FMain;
//  If you are using Delphi 6 Enterprise then disable this compiler directive
{$DEFINE MyXML}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ColorGrd, StdCtrls, ExtCtrls, Spin, CheckLst, Demo2Settings,
  PtSettings;

type
  TfrmMain = class(TForm)
    butExit: TButton;
    panUser: TGroupBox;
    labUserName: TLabel;
    edUserName: TEdit;
    labUserAge: TLabel;
    edUserAge: TSpinEdit;
    grpEmployments: TGroupBox;
    rgrEmployment: TRadioGroup;
    lstEmployments: TCheckListBox;
    panStorage: TGroupBox;
    butSaveAll: TButton;
    butLoadAll: TButton;
    butDeleteAll: TButton;
    chkEnumAsString: TCheckBox;
    chkINI: TRadioButton;
    chkReg: TRadioButton;
    chkXML: TRadioButton;
    procedure butExitClick(Sender: TObject);
    procedure butSaveAllClick(Sender: TObject);
    procedure butLoadAllClick(Sender: TObject);
    procedure butDeleteAllClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    function GetEmployment: TEmployment;
    function GetEmployments: TEmployments;
    procedure SetEmployment(const Value: TEmployment);
    procedure SetEmployments(const Value: TEmployments);
  protected
    function  CreateStorage: ISettingsStorage;
    procedure FetchSettings;
    procedure StoreSettings;
    procedure DeleteSettings;
  public
    property  Employment  : TEmployment   read GetEmployment  write SetEmployment;
    property  Employments : TEmployments  read GetEmployments write SetEmployments;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses
{$IFDEF MyXML}
  PtSettingsMyXML, MyXML,
{$ELSE}
  PtSettingsXML, XMLDoc,
{$ENDIF}
{$IFNDEF LINUX}
  PtSettingsReg,
{$ENDIF}
  PtSettingsIni, Demos;


{ TForm1 }

procedure TfrmMain.FormCreate(Sender: TObject);
begin
{$IFDEF LINUX}
  chkXML.Enabled  := false;
{$ENDIF}
end;

procedure TfrmMain.butExitClick(Sender: TObject);
begin
  Close;
end;

function TfrmMain.CreateStorage: ISettingsStorage;
var
  Storage : TSettingsStorage;
begin
  //  Create the desired type of storage
  if      chkINI.Checked  then
    Storage := TIniSettingsStorage.Create( FileName('.ini') )
{$IFNDEF LINUX}
  else if chkReg.Checked  then
    Storage := TRegSettingsStorage.Create( HKEY_LOCAL_MACHINE, 'SOFTWARE\PtSettings' )
{$ENDIF}
  else if chkXML.Checked  then
    Storage := TXMLSettingsStorage.Create( NewXMLDocument, FileName('.xml') )
  else
    raise Exception.Create('You have not selected a storage type');
  //  Indicate whether enumerations should be stored as strings or integers
  Storage.PreferStrings := chkEnumAsString.Checked;
  //  Return the new storage object
  Result  := Storage;
end;

procedure TfrmMain.DeleteSettings;
begin
  //  Delete settings from selected storage type
  if  chkReg.Checked  then  //  The registry storage is an exception.  See doc.
    CreateStorage.DeleteAll( Settings ) // This deletes the Settings object from the storage
  else
    CreateStorage.DeleteAll( nil ); // This deletes the file that the storage represents
end;

procedure TfrmMain.FetchSettings;
begin
  //  Load settings from desired storage type
  CreateStorage.LoadAll( Settings );
  //  Apply user settings
  edUserName.Text := Settings.User.Name;
  edUserAge.Value := Settings.User.Age;
  Employment      := Settings.User.Employment;
  Employments     := Settings.User.Employments;
end;

procedure TfrmMain.StoreSettings;
begin
  //  Set user settings
  Settings.User.Name        := edUserName.Text;
  Settings.User.Age         := edUserAge.Value;
  Settings.User.Employment  := Employment;
  Settings.User.Employments := Employments;
  //  Save settings to desired storage type
  CreateStorage.SaveAll( Settings );
end;

procedure TfrmMain.butSaveAllClick(Sender: TObject);
begin
  StoreSettings;
end;

procedure TfrmMain.butLoadAllClick(Sender: TObject);
begin
  FetchSettings;
end;

procedure TfrmMain.butDeleteAllClick(Sender: TObject);
begin
  DeleteSettings;
end;

//  Getter and setter methods for the properties Employment and Employments

function TfrmMain.GetEmployment: TEmployment;
begin
  Result  := TEmployment( rgrEmployment.ItemIndex )
end;

function TfrmMain.GetEmployments: TEmployments;
var
  e : TEmployment;
begin
  Result  := [];
  for e := Low(TEmployment) to High(TEmployment)  do
    if  lstEmployments.Checked[ integer(e) ]  then
      Result  := Result + [e]
end;

procedure TfrmMain.SetEmployment(const Value: TEmployment);
begin
  rgrEmployment.ItemIndex := integer( Value );
end;

procedure TfrmMain.SetEmployments(const Value: TEmployments);
var
  e : TEmployment;
begin
  for e := Low(TEmployment) to High(TEmployment)  do
    lstEmployments.Checked[ integer(e) ]  := e in Value
end;

end.
