unit Demo6Settings;

interface

uses
  PtSettings, PtEvents, Graphics;

type
  TSettingsEventGroup = class(TSettingsGroup)
  private
    fOnChange: TNotifyEvents;
  protected
    procedure Loading(Storage: ISettingsStorage); override;
    procedure Loaded(Storage: ISettingsStorage); override;
  public
    constructor Create(const AName: string; const AParent: TSettingsGroup = nil);
    destructor  Destroy; override;
    property  OnChange  : TNotifyEvents read fOnChange;
  end;

  TItemsSettings = class(TSettingsEventGroup)
  private
    fNumber: Integer;
    fColour: TColor;
    procedure SetNumber(const Value: Integer);
    procedure SetColour(const Value: TColor);
  published
    property  Number  : Integer read fNumber  write SetNumber;
    property  Colour  : TColor  read fColour  write SetColour;
  end;

  TThingsSettings = class(TSettingsEventGroup)
  private
    fCount: Integer;
    fName: String;
    procedure SetCount(const Value: Integer);
    procedure SetName(const Value: string);
  published
    property  Count : Integer read fCount write SetCount;
    property  Name  : string  read fName  write SetName;
  end;

  TSettings = class(TSettingsEventGroup)
  private
    fThings: TThingsSettings;
    fItems: TItemsSettings;
    procedure TotalChanged(Sender: TObject);
  public
    constructor Create(const AName: string; const AParent: TSettingsGroup = nil);
    destructor  Destroy; override;
    property  Things: TThingsSettings read fThings;
    property  Items: TItemsSettings read fItems;
    function  Total: integer;
  end;

var
  Settings: TSettings;
  Storage: ISettingsStorage;

implementation

uses
  PtSettingsIni, Demos, Windows;

procedure Initialize;
begin
  Settings := TSettings.Create('Demo6', nil);
  Storage := TIniSettingsStorage.Create( FileName('.ini') );
//  Storage := TRegSettingsStorage.Create( HKEY_LOCAL_MACHINE, '\SOFTWARE\PtSettings' );
  Storage.LoadAll( Settings );
end;

procedure Finalize;
begin
  Storage.SaveAll( Settings );
  Settings.Free;
end;

{ TSettingsEventGroup }

constructor TSettingsEventGroup.Create(const AName: string;
  const AParent: TSettingsGroup);
begin
  inherited;
  fOnChange := TNotifyEvents.Create(Self);
end;

destructor TSettingsEventGroup.Destroy;
begin
  fOnChange.Free;
  inherited;
end;

procedure TSettingsEventGroup.Loaded(Storage: ISettingsStorage);
begin
  inherited;
  OnChange.Enabled  := True;
  OnChange.Call;
end;

procedure TSettingsEventGroup.Loading(Storage: ISettingsStorage);
begin
  inherited;
  OnChange.Enabled  := False;
end;

{ TItemsSettings }

procedure TItemsSettings.SetColour(const Value: TColor);
begin
  fColour := Value;
  OnChange.Call;
end;

procedure TItemsSettings.SetNumber(const Value: Integer);
begin
  fNumber := Value;
  OnChange.Call;
end;

{ TThingsSettings }

procedure TThingsSettings.SetCount(const Value: Integer);
begin
  fCount := Value;
  OnChange.Call;
end;

procedure TThingsSettings.SetName(const Value: string);
begin
  fName := Value;
  OnChange.Call;
end;

{ TSettings }

constructor TSettings.Create(const AName: string; const AParent: TSettingsGroup);
begin
  inherited;
  fThings := TThingsSettings.Create('Things', Self);
  fItems := TItemsSettings.Create('Items', Self);
  Things.OnChange.Subscribe( TotalChanged );
  Items.OnChange.Subscribe( TotalChanged );
end;

destructor TSettings.Destroy;
begin
  Things.OnChange.Unsubscribe( TotalChanged );
  Items.OnChange.Unsubscribe( TotalChanged );
  inherited;
end;

function TSettings.Total: integer;
begin
  Result  := Things.Count + Items.Number
end;

procedure TSettings.TotalChanged(Sender: TObject);
begin
  OnChange.Call;
end;

initialization
  Initialize;

finalization
  Finalize;

end.

